@@ -10,6 +10,30 @@ __all__ = ["Controller"]
 
 log = tuned.logs.get()
 
+class TimerStore(object):
+	def __init__(self):
+		self._timers = dict()
+		self._timers_lock = threading.Lock()
+
+	def store_timer(self, token, timer):
+		with self._timers_lock:
+			self._timers[token] = timer
+
+	def drop_timer(self, token):
+		with self._timers_lock:
+			try:
+				timer = self._timers[token]
+				timer.cancel()
+				del self._timers[token]
+			except:
+				pass
+
+	def cancel_all(self):
+		with self._timers_lock:
+			for timer in self._timers.values():
+				timer.cancel()
+			self._timers.clear()
+
 class Controller(tuned.exports.interfaces.ExportableInterface):
 	"""
 	Controller's purpose is to keep the program running, start/stop the tuning,
@@ -17,11 +41,12 @@ class Controller(tuned.exports.interfaces.ExportableInterface):
 	"""
 
 	def __init__(self, daemon, global_config):
-		super(self.__class__, self).__init__()
+		super(Controller, self).__init__()
 		self._daemon = daemon
 		self._global_config = global_config
 		self._terminate = threading.Event()
 		self._cmd = commands()
+		self._timer_store = TimerStore()
 
 	def run(self):
 		"""
@@ -54,6 +79,32 @@ class Controller(tuned.exports.interfaces.ExportableInterface):
 	# identifying caller (with DBus it's the caller bus name) if authorized and empty
 	# string if not authorized, caller must be the last argument
 
+	def _log_capture_abort(self, token):
+		tuned.logs.log_capture_finish(token)
+		self._timer_store.drop_timer(token)
+
+	@exports.export("ii", "s")
+	def log_capture_start(self, log_level, timeout, caller = None):
+		if caller == "":
+			return ""
+		token = tuned.logs.log_capture_start(log_level)
+		if token is None:
+			return ""
+		if timeout > 0:
+			timer = threading.Timer(timeout,
+					self._log_capture_abort, args = [token])
+			self._timer_store.store_timer(token, timer)
+			timer.start()
+		return "" if token is None else token
+
+	@exports.export("s", "s")
+	def log_capture_finish(self, token, caller = None):
+		if caller == "":
+			return ""
+		res = tuned.logs.log_capture_finish(token)
+		self._timer_store.drop_timer(token)
+		return "" if res is None else res
+
 	@exports.export("", "b")
 	def start(self, caller = None):
 		if caller == "":
@@ -70,18 +121,26 @@ class Controller(tuned.exports.interfaces.ExportableInterface):
 		if caller == "":
 			return False
 		if not self._daemon.is_running():
-			return True
+			res = True
 		else:
-			return self._daemon.stop()
+			res = self._daemon.stop()
+		self._timer_store.cancel_all()
+		return res
 
 	@exports.export("", "b")
 	def reload(self, caller = None):
 		if caller == "":
 			return False
-		if not self._daemon.is_running():
+		if self._daemon.is_running():
+			stop_ok = self.stop()
+			if not stop_ok:
+				return False
+		try:
+			self._daemon.reload_profile_config()
+		except TunedException as e:
+			log.error("Failed to reload Tuned: %s" % e)
 			return False
-		else:
-			return self.stop() and self.start()
+		return self.start()
 
 	def _switch_profile(self, profile_name, manual):
 		was_running = self._daemon.is_running()
@@ -150,6 +209,12 @@ class Controller(tuned.exports.interfaces.ExportableInterface):
 		mode = consts.ACTIVE_PROFILE_MANUAL if manual else consts.ACTIVE_PROFILE_AUTO
 		return mode, ""
 
+	@exports.export("", "s")
+	def post_loaded_profile(self, caller = None):
+		if caller == "":
+			return ""
+		return self._daemon.post_loaded_profile or ""
+
 	@exports.export("", "b")
 	def disable(self, caller = None):
 		if caller == "":
@@ -157,7 +222,8 @@ class Controller(tuned.exports.interfaces.ExportableInterface):
 		if self._daemon.is_running():
 			self._daemon.stop()
 		if self._daemon.is_enabled():
-			self._daemon.set_profile(None, None, save_instantly=True)
+			self._daemon.set_all_profiles(None, True, None,
+						      save_instantly=True)
 		return True
 
 	@exports.export("", "b")
@@ -190,7 +256,7 @@ class Controller(tuned.exports.interfaces.ExportableInterface):
 	def recommend_profile(self, caller = None):
 		if caller == "":
 			return ""
-		return self._cmd.recommend_profile(hardcoded = not self._global_config.get_bool(consts.CFG_RECOMMEND_COMMAND, consts.CFG_DEF_RECOMMEND_COMMAND))
+		return self._daemon.profile_recommender.recommend()
 
 	@exports.export("", "b")
 	def verify_profile(self, caller = None):
@@ -203,3 +269,42 @@ class Controller(tuned.exports.interfaces.ExportableInterface):
 		if caller == "":
 			return False
 		return self._daemon.verify_profile(ignore_missing = True)
+
+	@exports.export("", "a{sa{ss}}")
+	def get_all_plugins(self, caller = None):
+		"""Return dictionary with accesible plugins
+
+		Return:
+		dictionary -- {plugin_name: {parameter_name: default_value}}
+		"""
+		if caller == "":
+			return False
+		plugins = {}
+		for plugin_class in self._daemon.get_all_plugins():
+			plugin_name = plugin_class.__module__.split(".")[-1].split("_", 1)[1]
+			conf_options = plugin_class._get_config_options()
+			plugins[plugin_name] = {}
+			for key, val in conf_options.items():
+				plugins[plugin_name][key] = str(val)
+		return plugins
+
+	@exports.export("s","s")
+	def get_plugin_documentation(self, plugin_name, caller = None):
+		"""Return docstring of plugin's class"""
+		if caller == "":
+			return False
+		return self._daemon.get_plugin_documentation(str(plugin_name))
+
+	@exports.export("s","a{ss}")
+	def get_plugin_hints(self, plugin_name, caller = None):
+		"""Return dictionary with plugin's parameters and their hints
+
+		Parameters:
+		plugin_name -- name of plugin
+
+		Return:
+		dictionary -- {parameter_name: hint}
+		"""
+		if caller == "":
+			return False
+		return self._daemon.get_plugin_hints(str(plugin_name))
