@@ -12,7 +12,13 @@ class DeviceMatcherUdev(device_matcher.DeviceMatcher):
 		"""
 
 		properties = ''
-		for key, val in list(device.items()):
+
+		try:
+			items = device.properties.items()
+		except AttributeError:
+			items = device.items()
+
+		for key, val in list(items):
 			properties += key + '=' + val + '\n'
 
 		return re.search(regex, properties, re.MULTILINE) is not None
